# Tenko parser test case

- Path: tests/testcases/for_statement/for-in/web_compat/combination_of_last_two_tests.md

> :: for statement : for-in : web compat
>
> ::> combination of last two tests
> 
> https://tc39.github.io/ecma262/#sec-iteration-statements
> 
> - for ( var ForBinding[?Yield, ?Await] in Expression[+In, ?Yield, ?Await] ) Statement[?Yield, ?Await, ?Return]
> 
> https://tc39.github.io/ecma262/#sec-initializers-in-forin-statement-heads
> 
> The following augments the IterationStatement;
> 
> - IterationStatement[Yield, Await, Return]:
>   - for ( var BindingIdentifier[?Yield, ?Await] Initializer[~In, ?Yield, ?Await] in Expression[+In, ?Yield, ?Await] ) Statement[?Yield, ?Await, ?Return]
> 
> (note that Expression also has comma-expression, there is no "Expressions" production)

## Input

`````js
for (var a = (++effects, -1) in stored = a, {a: 0, b: 1, c: 2});
`````

## Output

_Note: the whole output block is auto-generated. Manual changes will be overwritten!_

Below follow outputs in four parsing modes: sloppy mode, strict mode script goal, module goal, web compat mode (always sloppy).

Note that the output parts are auto-generated by the test runner to reflect actual result.

### Sloppy mode

Parsed with script goal and as if the code did not start with strict mode header.

`````
throws: Parser error!
  For-in and for-of binding can not have an init

for (var a = (++effects, -1) in stored = a, {a: 0, b: 1, c: 2});
                             ^------- error
`````

### Strict mode

Parsed with script goal but as if it was starting with `"use strict"` at the top.

_Output same as sloppy mode._

### Module goal

Parsed with the module goal.

_Output same as sloppy mode._

### Web compat mode

Parsed in sloppy script mode but with the web compat flag enabled.

`````
ast: {
  type: 'Program',
  loc:{start:{line:1,column:0},end:{line:1,column:64},source:''},
  body: [
    {
      type: 'ForInStatement',
      loc:{start:{line:1,column:0},end:{line:1,column:64},source:''},
      left: {
        type: 'VariableDeclaration',
        loc:{start:{line:1,column:5},end:{line:1,column:28},source:''},
        kind: 'var',
        declarations: [
          {
            type: 'VariableDeclarator',
            loc:{start:{line:1,column:9},end:{line:1,column:28},source:''},
            id: {
              type: 'Identifier',
              loc:{start:{line:1,column:9},end:{line:1,column:10},source:''},
              name: 'a'
            },
            init: {
              type: 'SequenceExpression',
              loc:{start:{line:1,column:14},end:{line:1,column:27},source:''},
              expressions: [
                {
                  type: 'UpdateExpression',
                  loc:{start:{line:1,column:14},end:{line:1,column:23},source:''},
                  argument: {
                    type: 'Identifier',
                    loc:{start:{line:1,column:16},end:{line:1,column:23},source:''},
                    name: 'effects'
                  },
                  operator: '++',
                  prefix: true
                },
                {
                  type: 'UnaryExpression',
                  loc:{start:{line:1,column:25},end:{line:1,column:27},source:''},
                  operator: '-',
                  prefix: true,
                  argument: {
                    type: 'Literal',
                    loc:{start:{line:1,column:26},end:{line:1,column:27},source:''},
                    value: 1,
                    raw: '1'
                  }
                }
              ]
            }
          }
        ]
      },
      right: {
        type: 'SequenceExpression',
        loc:{start:{line:1,column:32},end:{line:1,column:62},source:''},
        expressions: [
          {
            type: 'AssignmentExpression',
            loc:{start:{line:1,column:32},end:{line:1,column:42},source:''},
            left: {
              type: 'Identifier',
              loc:{start:{line:1,column:32},end:{line:1,column:38},source:''},
              name: 'stored'
            },
            operator: '=',
            right: {
              type: 'Identifier',
              loc:{start:{line:1,column:41},end:{line:1,column:42},source:''},
              name: 'a'
            }
          },
          {
            type: 'ObjectExpression',
            loc:{start:{line:1,column:44},end:{line:1,column:62},source:''},
            properties: [
              {
                type: 'Property',
                loc:{start:{line:1,column:45},end:{line:1,column:49},source:''},
                key: {
                  type: 'Identifier',
                  loc:{start:{line:1,column:45},end:{line:1,column:46},source:''},
                  name: 'a'
                },
                kind: 'init',
                method: false,
                computed: false,
                value: {
                  type: 'Literal',
                  loc:{start:{line:1,column:48},end:{line:1,column:49},source:''},
                  value: 0,
                  raw: '0'
                },
                shorthand: false
              },
              {
                type: 'Property',
                loc:{start:{line:1,column:51},end:{line:1,column:55},source:''},
                key: {
                  type: 'Identifier',
                  loc:{start:{line:1,column:51},end:{line:1,column:52},source:''},
                  name: 'b'
                },
                kind: 'init',
                method: false,
                computed: false,
                value: {
                  type: 'Literal',
                  loc:{start:{line:1,column:54},end:{line:1,column:55},source:''},
                  value: 1,
                  raw: '1'
                },
                shorthand: false
              },
              {
                type: 'Property',
                loc:{start:{line:1,column:57},end:{line:1,column:61},source:''},
                key: {
                  type: 'Identifier',
                  loc:{start:{line:1,column:57},end:{line:1,column:58},source:''},
                  name: 'c'
                },
                kind: 'init',
                method: false,
                computed: false,
                value: {
                  type: 'Literal',
                  loc:{start:{line:1,column:60},end:{line:1,column:61},source:''},
                  value: 2,
                  raw: '2'
                },
                shorthand: false
              }
            ]
          }
        ]
      },
      body: {
        type: 'EmptyStatement',
        loc:{start:{line:1,column:63},end:{line:1,column:64},source:''}
      }
    }
  ]
}

tokens (33x):
       ID_for PUNC_PAREN_OPEN ID_var IDENT PUNC_EQ PUNC_PAREN_OPEN
       PUNC_PLUS_PLUS IDENT PUNC_COMMA PUNC_MIN NUMBER_DEC
       PUNC_PAREN_CLOSE ID_in IDENT PUNC_EQ IDENT PUNC_COMMA
       PUNC_CURLY_OPEN IDENT PUNC_COLON NUMBER_DEC PUNC_COMMA IDENT
       PUNC_COLON NUMBER_DEC PUNC_COMMA IDENT PUNC_COLON NUMBER_DEC
       PUNC_CURLY_CLOSE PUNC_PAREN_CLOSE PUNC_SEMI
`````


## AST Printer

Printer output different from input [web]:

````js
for (var a = (++effects, - (1)) in ((stored = a), {a:0, b:1, c:2})) ;
````

Produces same AST
