# Tenko parser test case

- Path: tests/testcases/let_declaration/let_as_identifier_in_sloppy_mode/dynamic_prop_on_let_var/illegal_ambiguous_cases/in_arrow_expr_body.md

> :: let declaration : let as identifier in sloppy mode : dynamic prop on let var : illegal ambiguous cases
>
> ::> in arrow expr body
>
> (arrows are not strict by default) the non-block arrow body is an expression
>
> See section E: https://tc39.github.io/ecma262/#sec-additions-and-changes-that-introduce-incompatibilities-with-prior-editions
>
> > 13.2: In ECMAScript 2015, a StatementList beginning with the token let followed by the input elements LineTerminator then Identifier is the start of a LexicalDeclaration. In previous editions, automatic semicolon insertion would always insert a semicolon before the Identifier input element.
>
> > 13.5: In ECMAScript 2015, a StatementListItem beginning with the token let followed by the token [ is the start of a LexicalDeclaration. In previous editions such a sequence would be the start of an ExpressionStatement.
>
> > 13.7: In ECMAScript 2015, if the ( token of a for statement is immediately followed by the token sequence let [ then the let is treated as the start of a LexicalDeclaration. In previous editions such a token sequence would be the start of an Expression.
>
> > 13.7: In ECMAScript 2015, if the ( token of a for-in statement is immediately followed by the token sequence let [ then the let is treated as the start of a ForDeclaration. In previous editions such a token sequence would be the start of an LeftHandSideExpression.
>
> (note: the spec doesn't explicitly allow `let` as a var name but rather forbids it under certain situations. For example: in strict mode and as let/const names)
>
> A statement can not start with dynamic property access on `let` (`let[foo]=bar`) because it would be ambiguous with let destructuring. See note in
>
> https://tc39.github.io/ecma262/#prod-ExpressionStatement

## Input

`````js
_ => let[foo];
`````

## Output

_Note: the whole output block is auto-generated. Manual changes will be overwritten!_

Below follow outputs in four parsing modes: sloppy mode, strict mode script goal, module goal, web compat mode (always sloppy).

Note that the output parts are auto-generated by the test runner to reflect actual result.

### Sloppy mode

Parsed with script goal and as if the code did not start with strict mode header.

`````
ast: {
  type: 'Program',
  loc:{start:{line:1,column:0},end:{line:1,column:14},source:''},
  body: [
    {
      type: 'ExpressionStatement',
      loc:{start:{line:1,column:0},end:{line:1,column:14},source:''},
      expression: {
        type: 'ArrowFunctionExpression',
        loc:{start:{line:1,column:0},end:{line:1,column:13},source:''},
        params: [
          {
            type: 'Identifier',
            loc:{start:{line:1,column:0},end:{line:1,column:1},source:''},
            name: '_'
          }
        ],
        id: null,
        generator: false,
        async: false,
        expression: true,
        body: {
          type: 'MemberExpression',
          loc:{start:{line:1,column:5},end:{line:1,column:13},source:''},
          object: {
            type: 'Identifier',
            loc:{start:{line:1,column:5},end:{line:1,column:8},source:''},
            name: 'let'
          },
          property: {
            type: 'Identifier',
            loc:{start:{line:1,column:9},end:{line:1,column:12},source:''},
            name: 'foo'
          },
          computed: true
        }
      }
    }
  ]
}

tokens (8x):
       IDENT PUNC_EQ_GT ID_let PUNC_BRACKET_OPEN IDENT
       PUNC_BRACKET_CLOSE PUNC_SEMI
`````

### Strict mode

Parsed with script goal but as if it was starting with `"use strict"` at the top.

`````
throws: Parser error!
  Can not use `let` as variable name in strict mode

_ => let[foo];
        ^------- error
`````


### Module goal

Parsed with the module goal.

_Output same as strict mode._

### Web compat mode

Parsed in sloppy script mode but with the web compat flag enabled.

_Output same as sloppy mode._

## AST Printer

Printer output different from input [sloppy]:

````js
_ => ((let)[foo]);
````

Produces same AST
