# Tenko parser test case

- Path: tests/testcases/let_declaration/let_as_identifier_in_sloppy_mode/let_asi_block/illegal_ambiguous_cases/proper_case_with_confusing_newline.md

> :: let declaration : let as identifier in sloppy mode : let asi block : illegal ambiguous cases
>
> ::> proper case with confusing newline
>
> asi is only applied when the next token would lead to parse error; in this case the [ does not so it cannot parse this as prop-access
>
> See section E: https://tc39.github.io/ecma262/#sec-additions-and-changes-that-introduce-incompatibilities-with-prior-editions
>
> > 13.2: In ECMAScript 2015, a StatementList beginning with the token let followed by the input elements LineTerminator then Identifier is the start of a LexicalDeclaration. In previous editions, automatic semicolon insertion would always insert a semicolon before the Identifier input element.
>
> > 13.5: In ECMAScript 2015, a StatementListItem beginning with the token let followed by the token [ is the start of a LexicalDeclaration. In previous editions such a sequence would be the start of an ExpressionStatement.
>
> > 13.7: In ECMAScript 2015, if the ( token of a for statement is immediately followed by the token sequence let [ then the let is treated as the start of a LexicalDeclaration. In previous editions such a token sequence would be the start of an Expression.
>
> > 13.7: In ECMAScript 2015, if the ( token of a for-in statement is immediately followed by the token sequence let [ then the let is treated as the start of a ForDeclaration. In previous editions such a token sequence would be the start of an LeftHandSideExpression.
>
> (note: the spec doesn't explicitly allow `let` as a var name but rather forbids it under certain situations. For example: in strict mode and as let/const names)
> 
> While the spec doesn't mention it explicitly, I'm pretty sure the let-asi-block is illegal since it's equally ambigous as let-dynamic-prop.
> 
> https://tc39.github.io/ecma262/#prod-ExpressionStatement

## Input

`````js
let
{x} = x;
`````

## Output

_Note: the whole output block is auto-generated. Manual changes will be overwritten!_

Below follow outputs in four parsing modes: sloppy mode, strict mode script goal, module goal, web compat mode (always sloppy).

Note that the output parts are auto-generated by the test runner to reflect actual result.

### Sloppy mode

Parsed with script goal and as if the code did not start with strict mode header.

`````
ast: {
  type: 'Program',
  loc:{start:{line:1,column:0},end:{line:2,column:8},source:''},
  body: [
    {
      type: 'VariableDeclaration',
      loc:{start:{line:1,column:0},end:{line:2,column:8},source:''},
      kind: 'let',
      declarations: [
        {
          type: 'VariableDeclarator',
          loc:{start:{line:2,column:0},end:{line:2,column:7},source:''},
          id: {
            type: 'ObjectPattern',
            loc:{start:{line:2,column:0},end:{line:2,column:3},source:''},
            properties: [
              {
                type: 'Property',
                loc:{start:{line:2,column:1},end:{line:2,column:2},source:''},
                key: {
                  type: 'Identifier',
                  loc:{start:{line:2,column:1},end:{line:2,column:2},source:''},
                  name: 'x'
                },
                kind: 'init',
                method: false,
                computed: false,
                value: {
                  type: 'Identifier',
                  loc:{start:{line:2,column:1},end:{line:2,column:2},source:''},
                  name: 'x'
                },
                shorthand: true
              }
            ]
          },
          init: {
            type: 'Identifier',
            loc:{start:{line:2,column:6},end:{line:2,column:7},source:''},
            name: 'x'
          }
        }
      ]
    }
  ]
}

tokens (8x):
       ID_let PUNC_CURLY_OPEN IDENT PUNC_CURLY_CLOSE PUNC_EQ IDENT
       PUNC_SEMI
`````

### Strict mode

Parsed with script goal but as if it was starting with `"use strict"` at the top.

_Output same as sloppy mode._

### Module goal

Parsed with the module goal.

_Output same as sloppy mode._

### Web compat mode

Parsed in sloppy script mode but with the web compat flag enabled.

_Output same as sloppy mode._

## AST Printer

Printer output different from input [sloppy]:

````js
let {x} = x;
````

Produces same AST
