# Tenko parser test case

- Path: tests/testcases/let_declaration/while_let_newline_objlit.md

> :: let declaration
>
> ::> while let newline objlit
>
> The empty object literal can still be a pattern
>
> https://twitter.com/Ghost1240145716/status/1153227444104491008
>
> This is valid in sloppy because the while CANT have a `let` declaration so it must be an ident expression.
>
> After the ident ASI triggers as usual and the `{}` is parsed as an empty block.
>
> In strict it can't be an ident so it just throws an error.

## Input

`````js
while (x) let
{}
`````

## Output

_Note: the whole output block is auto-generated. Manual changes will be overwritten!_

Below follow outputs in four parsing modes: sloppy mode, strict mode script goal, module goal, web compat mode (always sloppy).

Note that the output parts are auto-generated by the test runner to reflect actual result.

### Sloppy mode

Parsed with script goal and as if the code did not start with strict mode header.

`````
ast: {
  type: 'Program',
  loc:{start:{line:1,column:0},end:{line:2,column:2},source:''},
  body: [
    {
      type: 'WhileStatement',
      loc:{start:{line:1,column:0},end:{line:1,column:13},source:''},
      test: {
        type: 'Identifier',
        loc:{start:{line:1,column:7},end:{line:1,column:8},source:''},
        name: 'x'
      },
      body: {
        type: 'ExpressionStatement',
        loc:{start:{line:1,column:10},end:{line:1,column:13},source:''},
        expression: {
          type: 'Identifier',
          loc:{start:{line:1,column:10},end:{line:1,column:13},source:''},
          name: 'let'
        }
      }
    },
    {
      type: 'BlockStatement',
      loc:{start:{line:2,column:0},end:{line:2,column:2},source:''},
      body: []
    }
  ]
}

tokens (9x):
       ID_while PUNC_PAREN_OPEN IDENT PUNC_PAREN_CLOSE ID_let ASI
       PUNC_CURLY_OPEN PUNC_CURLY_CLOSE
`````

### Strict mode

Parsed with script goal but as if it was starting with `"use strict"` at the top.

`````
throws: Parser error!
  `let` declaration not allowed here and `let` cannot be a regular var or label name in strict mode

while (x) let
{}
^------- error
`````


### Module goal

Parsed with the module goal.

_Output same as strict mode._

### Web compat mode

Parsed in sloppy script mode but with the web compat flag enabled.

_Output same as sloppy mode._

## AST Printer

Printer output different from input [sloppy]:

````js
while (x) (let);
{}
````

Produces same AST
