# Tenko parser test case

- Path: tests/testcases/tests_related_to_bindings/block/annex_b_function_statement_exception/label_func_and_var_in_func.md

> :: tests related to bindings : block : annex b function statement exception
>
> ::> label func and var in func
>
>> Labels pass the lex decl of function statements through to the parent scope so this should fail with lex/var dupe decl.

https://tc39.es/ecma262/#sec-labelled-statements-static-semantics-early-errors

> It is a Syntax Error if any source text matches this rule.

https://tc39.es/ecma262/#sec-labelled-function-declarations

> It is a Syntax Error if any strict mode source code matches this rule.

> The early error rules for WithStatement, IfStatement, and IterationStatement prevent these statements from containing a labelled FunctionDeclaration in non-strict code.

https://tc39.es/ecma262/#sec-labelled-statements-static-semantics-lexicallydeclarednames

> Return BoundNames of FunctionDeclaration.

So (in web compat only) the lex names of a labelled function statement is its name (same rules as plain function decls). So this is a lex name and it should clash with the var that was defined before.

In a function decl the func name should not propagate to the parent and so this should not clash, in any mode.


## Input

`````js
function g(){
  var f = 123;
  oops: function f(){}
}
`````

## Output

_Note: the whole output block is auto-generated. Manual changes will be overwritten!_

Below follow outputs in four parsing modes: sloppy mode, strict mode script goal, module goal, web compat mode (always sloppy).

Note that the output parts are auto-generated by the test runner to reflect actual result.

### Sloppy mode

Parsed with script goal and as if the code did not start with strict mode header.

`````
throws: Parser error!
  A "labelled function declaration" is only allowed in sloppy web compat mode

function g(){
  var f = 123;
  oops: function f(){}
                 ^------- error

}
`````

### Strict mode

Parsed with script goal but as if it was starting with `"use strict"` at the top.

_Output same as sloppy mode._

### Module goal

Parsed with the module goal.

_Output same as sloppy mode._

### Web compat mode

Parsed in sloppy script mode but with the web compat flag enabled.

`````
ast: {
  type: 'Program',
  loc:{start:{line:1,column:0},end:{line:4,column:1},source:''},
  body: [
    {
      type: 'FunctionDeclaration',
      loc:{start:{line:1,column:0},end:{line:4,column:1},source:''},
      generator: false,
      async: false,
      id: {
        type: 'Identifier',
        loc:{start:{line:1,column:9},end:{line:1,column:10},source:''},
        name: 'g'
      },
      params: [],
      body: {
        type: 'BlockStatement',
        loc:{start:{line:1,column:12},end:{line:4,column:1},source:''},
        body: [
          {
            type: 'VariableDeclaration',
            loc:{start:{line:2,column:2},end:{line:2,column:14},source:''},
            kind: 'var',
            declarations: [
              {
                type: 'VariableDeclarator',
                loc:{start:{line:2,column:6},end:{line:2,column:13},source:''},
                id: {
                  type: 'Identifier',
                  loc:{start:{line:2,column:6},end:{line:2,column:7},source:''},
                  name: 'f'
                },
                init: {
                  type: 'Literal',
                  loc:{start:{line:2,column:10},end:{line:2,column:13},source:''},
                  value: 123,
                  raw: '123'
                }
              }
            ]
          },
          {
            type: 'LabeledStatement',
            loc:{start:{line:3,column:2},end:{line:3,column:22},source:''},
            label: {
              type: 'Identifier',
              loc:{start:{line:3,column:2},end:{line:3,column:6},source:''},
              name: 'oops'
            },
            body: {
              type: 'FunctionDeclaration',
              loc:{start:{line:3,column:8},end:{line:3,column:22},source:''},
              generator: false,
              async: false,
              id: {
                type: 'Identifier',
                loc:{start:{line:3,column:17},end:{line:3,column:18},source:''},
                name: 'f'
              },
              params: [],
              body: {
                type: 'BlockStatement',
                loc:{start:{line:3,column:20},end:{line:3,column:22},source:''},
                body: []
              }
            }
          }
        ]
      }
    }
  ]
}

tokens (20x):
       ID_function IDENT PUNC_PAREN_OPEN PUNC_PAREN_CLOSE
       PUNC_CURLY_OPEN ID_var IDENT PUNC_EQ NUMBER_DEC PUNC_SEMI IDENT
       PUNC_COLON ID_function IDENT PUNC_PAREN_OPEN PUNC_PAREN_CLOSE
       PUNC_CURLY_OPEN PUNC_CURLY_CLOSE PUNC_CURLY_CLOSE
`````


## AST Printer

Printer output different from input [web]:

````js
function g() {var f = 123;
oops: function f() {}}
````

Produces same AST
